@@ -19,7 +19,7 @@
 LOCALE=[ENV['PWD'] + "/locale", "/usr/local/share/locale"]
 LOCALE.each{|dir| if File.directory?(dir) ; ENV['GETTEXT_PATH'] = dir ; break end}
 
-$rr_version = '0.6.0' #application wide setting
+$rr_version = '0.6.2' #application wide setting
 
 begin
 	require 'gettext'
@@ -1175,10 +1175,11 @@ attr_accessor :artist, :album, :genre, :year, :tracklist, :varArtists, :discNumb
 		
 		@query = @url.path + "?cmd=cddb+query+" + CGI.escape("#{@disc.freedbString.chomp}") + "&hello=" + 
 			CGI.escape("#{@freedbSettings['username']} #{@freedbSettings['hostname']} rubyripper #{$rr_version}") + "&proto=6"
+    puts "cddb query 1 = #{@query}" if @settings['debug']
 		if @verbose ; puts "Created query string: #{@query}" end
 
 		begin
-			respons, @answer = @server.get(@query)
+			@answer = @server.get(@query).body
 			requestDisc()
 		rescue
 			puts "Exception thrown: #{$!}"
@@ -1227,9 +1228,10 @@ attr_accessor :artist, :album, :genre, :year, :tracklist, :varArtists, :discNumb
 	def rawResponse #Retrieve all usefull metadata into @rawResponse
 		@query = @url.path + "?cmd=cddb+read+" + CGI.escape("#{@category} #{@discid}") + "&hello=" + 
 			CGI.escape("#{@freedbSettings['username']} #{@freedbSettings['hostname']} rubyripper #{$rr_version}") + "&proto=6"
+    puts "cddb query 2 = #{@query}" if @settings['debug']
 		if @verbose ; puts "Created fetch string: #{@query}" end
 		
-		response, answer = @server.get(@query)
+		answer = @server.get(@query).body
 		answers = answer.split("\n")
 		answers.each do |line|
 			line.chomp!
@@ -1320,7 +1322,7 @@ attr_accessor :artist, :album, :genre, :year, :tracklist, :varArtists, :discNumb
 
 		if varArtist == true
 			@backupTracklist = @tracklist.dup() #backup before overwrite with new values
-			@tracklist.each_index{|index| @varArtists[index], @tracklist[index] = @tracklist[index].split(/\s*[-\/]|(\sby\s)\s*/)} #remove any spaces (\s) around sep
+			@tracklist.each_index{|index| @varArtists[index], @tracklist[index] = @tracklist[index].split(/\s*[\/-]\s*|(\sby\s)\s*/)} #remove any spaces (\s) around sep
 		end
 	end
 	
@@ -1771,8 +1773,14 @@ attr_reader :status, :artist, :album, :year, :genre
 	end
 end
 
+require 'fileutils'
+
 class SecureRip
 	attr_writer :cancelled 
+
+	BYTES_WAV_CONTAINER = 44 # wav container overhead
+	BYTES_AUDIO_SECTOR = 2352 # size for a audiocd sector as used in cdparanoia
+	BYTES_SECTOR_GROUP = 1024 * BYTES_AUDIO_SECTOR # compare groups first due performance issue ruby 1.8
 	
 	def initialize(settings, encoding)
 		@settings = settings
@@ -1835,7 +1843,7 @@ class SecureRip
 	# when sox is finished move it back to the original name
 	def deEmphasize(track)
 		if @settings['create_cue'] && @settings['preEmphasis'] == "sox" &&
-			@settings['cd'].toc.hasPreEmph(track) && installed(sox)
+			@settings['cd'].toc.hasPreEmph(track) && installed('sox')
 			`sox #{@settings['Out'].getTempFile(track, 1)} #{@settings['Out'].getTempFile(track, 2)}`
 			if $?.success?
 				FileUtils.mv(@settings['Out'].getTempFile(track, 2), @settings['Out'].getTempFile(track, 1))
@@ -1887,7 +1895,7 @@ class SecureRip
 		@settings['log'].ripPerc(@progress)
 		return true
 	end
-	
+
 	def doNewTrial(track)
 		fileOk = false
 	
@@ -1940,27 +1948,13 @@ class SecureRip
 		return true
 	end
 
+	# Start and close the first file comparisons
 	def analyzeFiles(track)
-		@settings['log'].add(_("Analyzing files for mismatching chunks\n"))
-		files = Array.new
-		@reqMatchesAll.times do |time|
-			files << File.new(@settings['Out'].getTempFile(track, time + 1), 'r')
-		end
-				
-		(@reqMatchesAll - 1).times do |time|
-			index = 0 ; files.each{|file| file.pos = 44} # 44 = wav container overhead, 2352 = size for a audiocd sector as used in cdparanoia
-			while index + 44 < @settings['cd'].getFileSize(track)
-				if !@errors.key?(index) && files[0].read(2352) != files[time + 1].read(2352) # Does this sector matches the previous ones? and isn't the position already known?
-					files.each{|file| file.pos = index + 44} # Reset each read position of the files
-					@errors[index] = Array.new
-					files.each{|file| @errors[index] << file.read(2352)} # Save the chunk for all files in the just created array
-				end
-				index += 2352
-			end
-		end
-		
-		files.each{|file| file.close}
-		
+		start = Time.now()
+		@settings['log'].add(_("Analyzing files for mismatching chunks"))
+		compareSectors(track)
+		@settings['log'].add(_(" (%s second(s))\n") %[(Time.now - start).to_i])
+
 		# Remove the files now we analyzed them. Differences are saved in memory.
 		(@reqMatchesAll - 1).times{|time| File.delete(@settings['Out'].getTempFile(track, time + 2))}
  
@@ -1971,6 +1965,66 @@ class SecureRip
 			@settings['log'].add(_("%s chunk(s) didn't match %s times.\n") % [@errors.length, @reqMatchesAll])
 		end
 	end
+
+	# Compare a range of sectors within two files
+	def compareSectorRange(files, fileIndexA, fileIndexB, sectorOffset)
+
+		# First do one large block compare, and bail out if the same.
+		# A lot faster than sector by sector comparison on some Ruby implementations
+		# (http://code.google.com/p/rubyripper/issues/detail?id=348).
+		fa = files[fileIndexA]
+		fb = files[fileIndexB]
+		fa.sysseek(BYTES_WAV_CONTAINER + sectorOffset, IO::SEEK_SET)
+		fb.sysseek(BYTES_WAV_CONTAINER + sectorOffset, IO::SEEK_SET)
+		if fa.sysread(BYTES_SECTOR_GROUP) == fb.sysread(BYTES_SECTOR_GROUP)
+			return
+		end
+
+		# There was a difference, so drill down and find the individual sectors
+		pos = sectorOffset
+		endPos = pos + BYTES_SECTOR_GROUP
+    begin
+		while pos < endPos && pos < @endSectorOffset
+			# If we haven't already recorded an error for this sector
+			if !@errors.key?(pos)
+				# Is there a mismatch
+				fa.sysseek(BYTES_WAV_CONTAINER + pos, IO::SEEK_SET)
+				fb.sysseek(BYTES_WAV_CONTAINER + pos, IO::SEEK_SET)
+				if fa.sysread(BYTES_AUDIO_SECTOR) != fb.sysread(BYTES_AUDIO_SECTOR)
+					# Store a copy of the sector from each file in the error map
+					files.each{|file| file.sysseek(BYTES_WAV_CONTAINER + pos, IO::SEEK_SET)}
+					@errors[pos] = Array.new
+					files.each{|file| @errors[pos] << file.sysread(BYTES_AUDIO_SECTOR)}
+				end
+			end
+			pos += BYTES_AUDIO_SECTOR
+		end
+    rescue EOFError
+      puts "An unexpected end-of-file error occured for position #{pos}."
+      puts "Final sector should start at #{@endSectorOffset}."
+      puts "This sector is now ignored for comparison!!"
+    end
+	end
+
+	# Compare each temp file, recording all sectors which don't match in @errors
+	def compareSectors(track)
+		files = Array.new
+		@reqMatchesAll.times do |time|
+			files << File.new(@settings['Out'].getTempFile(track, time + 1), 'r')
+		end
+
+		@endSectorOffset = @settings['cd'].getFileSize(track) - BYTES_WAV_CONTAINER
+
+		(@reqMatchesAll - 1).times do |time|
+			sectorOffset = 0
+			while sectorOffset < @endSectorOffset
+				compareSectorRange(files, 0, time + 1, sectorOffset)
+				sectorOffset += BYTES_SECTOR_GROUP
+			end
+		end
+		
+		files.each{|file| file.close}
+	end
 	
 	# When required matches for mismatched sectors are bigger than there are 
 	# trials to be tested, readErrorPos() just reads the mismatched sectors
@@ -1981,8 +2035,8 @@ class SecureRip
 	def readErrorPos(track)
 		file = File.new(@settings['Out'].getTempFile(track, @trial), 'r')
 		@errors.keys.sort.each do |start_chunk|
-			file.pos = start_chunk + 44
-			@errors[start_chunk] << file.read(2352)
+			file.sysseek(start_chunk + BYTES_WAV_CONTAINER, IO::SEEK_SET)
+			@errors[start_chunk] << file.sysread(BYTES_AUDIO_SECTOR)
 		end
 		file.close
 
@@ -2004,15 +2058,15 @@ class SecureRip
 		
 		# Sort the hash keys to prevent jumping forward and backwards in the file
 		@errors.keys.sort.each do |start_chunk|
-			file2.pos = start_chunk + 44
-			@errors[start_chunk] << temp = file2.read(2352)
+			file2.sysseek(start_chunk + BYTES_WAV_CONTAINER, IO::SEEK_SET)
+			@errors[start_chunk] << temp = file2.sysread(BYTES_AUDIO_SECTOR)
 
 			# now sort the array and see if the new read value has enough matches
 			# right index minus left index of the read value is amount of matches
 			@errors[start_chunk].sort!
 			if (@errors[start_chunk].rindex(temp) - @errors[start_chunk].index(temp)) == (@reqMatchesErrors - 1)
-				file1.pos = start_chunk + 44
-				file1.write(temp)
+				file1.sysseek(start_chunk + BYTES_WAV_CONTAINER, IO::SEEK_SET)
+				file1.syswrite(temp)
 				@errors.delete(start_chunk)
 			end
 		end
@@ -2084,7 +2138,7 @@ class SecureRip
 		file = File.open(@settings['Out'].getTempFile(track, 1), 'r')
 		index = 0
 		while (index < @settings['cd'].getFileSize(track))
-			digest << file.read(100000)
+			digest << file.sysread(100000)
 			index += 100000
 		end
 		file.close()
